import boto3
import os
from typing import Optional

class SNSClient:
    def __init__(self):
        self.sns_client = boto3.client('sns',region_name=os.environ.get("AWS_REGION", "us-east-1"))
        self.topic_arn = os.getenv('SNS_TOPIC_ARN')

    def create_appointment_topic(self):
        """Create the appointments notification topic"""
        response = self.sns_client.create_topic(Name='AppointmentNotifications')
        self.topic_arn = response['TopicArn']
        return self.topic_arn

    def subscribe_patient(self, email: str):
        """Subscribe a patient to notifications"""
        return self.sns_client.subscribe(
            TopicArn=self.topic_arn,
            Protocol='email',
            Endpoint=email
        )

    def send_appointment_notification(self, email: str, message: str):
        """Send appointment status notification"""
        self.sns_client.publish(
            TopicArn=self.topic_arn,
            Message=message,
            Subject='Appointment Status Update',
            MessageAttributes={
                'email': {
                    'DataType': 'String',
                    'StringValue': email
                }
            }
        )

    def send_prescription_notification(self, patient_email:str, message:str):
        self.sns_client.publish(
            TopicArn=self.topic_arn,
            Message=message,
            Subject='Prescription Upload',
            MessageAttributes={
                'email': {
                    'DataType': 'String',
                    'StringValue': patient_email
                }
            }
        )


from .client import SNSClient

class AppointmentNotification:
    def __init__(self):
        self.sns_client = SNSClient()
        if not self.sns_client.topic_arn:
            self.sns_client.create_appointment_topic()

    def send_approval_notification(self, patient_email: str, doctor_name: str, appointment_date: str, appointment_time: str):
        message = f"""Your appointment with Dr. {doctor_name} 
                    on {appointment_date} at {appointment_time} 
                    has been approved!"""
        self.sns_client.send_appointment_notification(patient_email, message)

    def send_rejection_notification(self, patient_email: str, doctor_name: str, 
                                  appointment_date: str, appointment_time: str, 
                                  reason: str):
        message = f"""Your appointment with Dr. {doctor_name} 
                    on {appointment_date} at {appointment_time} 
                    has been rejected. Reason: {reason}"""
        self.sns_client.send_appointment_notification(patient_email, message)

    def publish_uploaded_perscription(self, patient_email:str):
        message = f"Prescription uploaded for patient {patient_email}"
        self.sns_client.send_prescription_notification(patient_email,message)


import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="sns-appointment",
    version="0.2.0",
    author="Jeena",
    author_email="jeena7758@gmail.com",
    description="A library for sending AWS SNS notifications for doctor appointments",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Jeena7758",
    packages=setuptools.find_packages(),
    install_requires=[
        'boto3>=1.17.0',
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)